@@ -13,14 +13,8 @@ use Cwd;
 use vars qw( $VERSION @ISA );
 
 BEGIN {
-    $VERSION = '1.31_04';
+    $VERSION = '1.21';
     @ISA     = qw( Archive::Zip );
-
-    if ( $^O eq 'MSWin32' ) {
-        require Win32;
-        require Encode;
-        Encode->import( qw{ encode_utf8 decode_utf8 } );
-    }
 }
 
 use Archive::Zip qw(
@@ -30,8 +24,6 @@ use Archive::Zip qw(
   :UTILITY_METHODS
 );
 
-our $UNICODE;
-
 # Note that this returns undef on read errors, else new zip object.
 
 sub new {
@@ -54,19 +46,13 @@ sub new {
         $class
     );
     $self->{'members'} = [];
-    my $fileName = ( ref( $_[0] ) eq 'HASH' ) ? shift->{filename} : shift;
-    if ($fileName) {
-        my $status = $self->read($fileName);
+    if (@_) {
+        my $status = $self->read(@_);
         return $status == AZ_OK ? $self : undef;
     }
     return $self;
 }
 
-sub storeSymbolicLink {
-    my $self = shift;
-    $self->{'storeSymbolicLink'} = shift;
-}
-
 sub members {
     @{ shift->{'members'} };
 }
@@ -82,8 +68,7 @@ sub memberNames {
 
 # return ref to member with given name or undef
 sub memberNamed {
-    my $self     = shift;
-    my $fileName = ( ref( $_[0] ) eq 'HASH' ) ? shift->{zipName} : shift;
+    my ( $self, $fileName ) = @_;
     foreach my $member ( $self->members() ) {
         return $member if $member->fileName() eq $fileName;
     }
@@ -91,8 +76,7 @@ sub memberNamed {
 }
 
 sub membersMatching {
-    my $self    = shift;
-    my $pattern = ( ref( $_[0] ) eq 'HASH' ) ? shift->{regex} : shift;
+    my ( $self, $pattern ) = @_;
     return grep { $_->fileName() =~ /$pattern/ } $self->members();
 }
 
@@ -124,8 +108,7 @@ sub zipfileComment {
     my $self    = shift;
     my $comment = $self->{'zipfileComment'};
     if (@_) {
-        my $new_comment = ( ref( $_[0] ) eq 'HASH' ) ? shift->{comment} : shift;
-        $self->{'zipfileComment'} = pack( 'C0a*', $new_comment );    # avoid Unicode
+        $self->{'zipfileComment'} = pack( 'C0a*', shift() );    # avoid unicode
     }
     return $comment;
 }
@@ -140,8 +123,7 @@ sub fileName {
 }
 
 sub removeMember {
-    my $self    = shift;
-    my $member  = ( ref( $_[0] ) eq 'HASH' ) ? shift->{memberOrZipName} : shift;
+    my ( $self, $member ) = @_;
     $member = $self->memberNamed($member) unless ref($member);
     return undef unless $member;
     my @newMembers = grep { $_ != $member } $self->members();
@@ -150,17 +132,7 @@ sub removeMember {
 }
 
 sub replaceMember {
-    my $self = shift;
-
-    my ( $oldMember, $newMember );
-    if ( ref( $_[0] ) eq 'HASH' ) {
-        $oldMember = $_[0]->{memberOrZipName};
-        $newMember = $_[0]->{newMember};
-    }
-    else {
-        ( $oldMember, $newMember ) = @_;
-    }
-
+    my ( $self, $oldMember, $newMember ) = @_;
     $oldMember = $self->memberNamed($oldMember) unless ref($oldMember);
     return undef unless $oldMember;
     return undef unless $newMember;
@@ -171,20 +143,12 @@ sub replaceMember {
 }
 
 sub extractMember {
-    my $self = shift;
-
-    my ( $member, $name );
-    if ( ref( $_[0] ) eq 'HASH' ) {
-        $member = $_[0]->{memberOrZipName};
-        $name   = $_[0]->{name};
-    }
-    else {
-        ( $member, $name ) = @_;
-    }
-
+    my $self   = shift;
+    my $member = shift;
     $member = $self->memberNamed($member) unless ref($member);
     return _error('member not found') unless $member;
     my $originalSize = $member->compressedSize();
+    my $name         = shift;                       # local FS name if given
     my ( $volumeName, $dirName, $fileName );
     if ( defined($name) ) {
         ( $volumeName, $dirName, $fileName ) = File::Spec->splitpath($name);
@@ -208,21 +172,13 @@ sub extractMember {
 }
 
 sub extractMemberWithoutPaths {
-    my $self = shift;
-
-    my ( $member, $name );
-    if ( ref( $_[0] ) eq 'HASH' ) {
-        $member = $_[0]->{memberOrZipName};
-        $name   = $_[0]->{name};
-    }
-    else {
-        ( $member, $name ) = @_;
-    }
-
+    my $self   = shift;
+    my $member = shift;
     $member = $self->memberNamed($member) unless ref($member);
     return _error('member not found') unless $member;
     my $originalSize = $member->compressedSize();
     return AZ_OK if $member->isDirectory();
+    my $name = shift;
     unless ($name) {
         $name = $member->fileName();
         $name =~ s{.*/}{};    # strip off directories, if any
@@ -234,127 +190,43 @@ sub extractMemberWithoutPaths {
 }
 
 sub addMember {
-    my $self       = shift;
-    my $newMember  = ( ref( $_[0] ) eq 'HASH' ) ? shift->{member} : shift;
+    my ( $self, $newMember ) = @_;
     push( @{ $self->{'members'} }, $newMember ) if $newMember;
     return $newMember;
 }
 
 sub addFile {
-    my $self = shift;
-
-    my ( $fileName, $newName, $compressionLevel );
-    if ( ref( $_[0] ) eq 'HASH' ) {
-        $fileName         = $_[0]->{filename};
-        $newName          = $_[0]->{zipName};
-        $compressionLevel = $_[0]->{compressionLevel};
-    }
-    else {
-        ( $fileName, $newName, $compressionLevel ) = @_;
-    }
-
-    if ( $^O eq 'MSWin32' && $Archive::Zip::UNICODE ) {
-        $fileName = Win32::GetANSIPathName($fileName);
-    }
-
+    my $self      = shift;
+    my $fileName  = shift;
+    my $newName   = shift;
     my $newMember = $self->ZIPMEMBERCLASS->newFromFile( $fileName, $newName );
-    $newMember->desiredCompressionLevel($compressionLevel);
-    if ( $self->{'storeSymbolicLink'} && -l $fileName ) {
-        my $newMember = $self->ZIPMEMBERCLASS->newFromString(readlink $fileName, $newName);
-        # For symbolic links, External File Attribute is set to 0xA1FF0000 by Info-ZIP
-        $newMember->{'externalFileAttributes'} = 0xA1FF0000;
-        $self->addMember($newMember);
-    } else {
-        $self->addMember($newMember);
-    }
-    if ( $^O eq 'MSWin32' && $Archive::Zip::UNICODE ) {
-        $newMember->{'fileName'} = encode_utf8( Win32::GetLongPathName($fileName) );
-    }
+    $self->addMember($newMember) if defined($newMember);
     return $newMember;
 }
 
 sub addString {
-    my $self = shift;
-
-    my ( $stringOrStringRef, $name, $compressionLevel );
-    if ( ref( $_[0] ) eq 'HASH' ) {
-        $stringOrStringRef = $_[0]->{string};
-        $name              = $_[0]->{zipName};
-        $compressionLevel  = $_[0]->{compressionLevel};
-    }
-    else {
-        ( $stringOrStringRef, $name, $compressionLevel ) = @_;;
-    }
-
-    my $newMember = $self->ZIPMEMBERCLASS->newFromString(
-        $stringOrStringRef, $name
-    );
-    $newMember->desiredCompressionLevel($compressionLevel);
+    my $self      = shift;
+    my $newMember = $self->ZIPMEMBERCLASS->newFromString(@_);
     return $self->addMember($newMember);
 }
 
 sub addDirectory {
-    my $self = shift;
-
-    my ( $name, $newName );
-    if ( ref( $_[0] ) eq 'HASH' ) {
-        $name    = $_[0]->{directoryName};
-        $newName = $_[0]->{zipName};
-    }
-    else {
-        ( $name, $newName ) = @_;
-    }
-
-    if ( $^O eq 'MSWin32' && $Archive::Zip::UNICODE ) {
-        $name = Win32::GetANSIPathName($name);
-    }
-
+    my ( $self, $name, $newName ) = @_;
     my $newMember = $self->ZIPMEMBERCLASS->newDirectoryNamed( $name, $newName );
-    if ( $self->{'storeSymbolicLink'} && -l $name ) {
-        my $link = readlink $name;
-        ( $newName =~ s{/$}{} ) if $newName; # Strip trailing /
-        my $newMember = $self->ZIPMEMBERCLASS->newFromString($link, $newName);
-        # For symbolic links, External File Attribute is set to 0xA1FF0000 by Info-ZIP
-        $newMember->{'externalFileAttributes'} = 0xA1FF0000;
-        $self->addMember($newMember);
-    } else {
-        $self->addMember($newMember);
-    }
-    if ( $^O eq 'MSWin32' && $Archive::Zip::UNICODE ) {
-        $newMember->{'fileName'} = encode_utf8( Win32::GetLongPathName($name) );
-    }
+    $self->addMember($newMember);
     return $newMember;
 }
 
 # add either a file or a directory.
 
 sub addFileOrDirectory {
-    my $self = shift;
-
-    my ( $name, $newName, $compressionLevel );
-    if ( ref( $_[0] ) eq 'HASH' ) {
-        $name             = $_[0]->{name};
-        $newName          = $_[0]->{zipName};
-        $compressionLevel = $_[0]->{compressionLevel};
-    }
-    else {
-        ( $name, $newName, $compressionLevel ) = @_;
-    }
-
-    if ( $^O eq 'MSWin32' && $Archive::Zip::UNICODE ) {
-        $name = Win32::GetANSIPathName($name);
-    }
-
-    $name =~ s{/$}{};
-    if ( $newName ) {
-        $newName =~ s{/$}{};
-    } else {
-        $newName = $name;
-    }
+    my ( $self, $name, $newName ) = @_;
     if ( -f $name ) {
-        return $self->addFile( $name, $newName, $compressionLevel );
+        ( $newName =~ s{/$}{} ) if $newName;
+        return $self->addFile( $name, $newName );
     }
     elsif ( -d $name ) {
+        ( $newName =~ s{[^/]$}{&/} ) if $newName;
         return $self->addDirectory( $name, $newName );
     }
     else {
@@ -363,17 +235,7 @@ sub addFileOrDirectory {
 }
 
 sub contents {
-    my $self = shift;
-
-    my ( $member, $newContents );
-    if ( ref( $_[0] ) eq 'HASH' ) {
-        $member      = $_[0]->{memberOrZipName};
-        $newContents = $_[0]->{contents};
-    }
-    else {
-        ( $member, $newContents ) = @_;
-    }
-
+    my ( $self, $member, $newContents ) = @_;
     return _error('No member name given') unless $member;
     $member = $self->memberNamed($member) unless ref($member);
     return undef unless $member;
@@ -381,9 +243,8 @@ sub contents {
 }
 
 sub writeToFileNamed {
-    my $self = shift;
-    my $fileName =
-      ( ref( $_[0] ) eq 'HASH' ) ? shift->{filename} : shift;  # local FS format
+    my $self     = shift;
+    my $fileName = shift;    # local FS format
     foreach my $member ( $self->members() ) {
         if ( $member->_usesFileNamed($fileName) ) {
             return _error( "$fileName is needed by member "
@@ -404,25 +265,16 @@ sub writeToFileNamed {
 # perhaps to make a self-extracting archive.
 sub writeToFileHandle {
     my $self = shift;
-
-    my ( $fh, $fhIsSeekable );
-    if ( ref( $_[0] ) eq 'HASH' ) {
-        $fh           = $_[0]->{fileHandle};
-        $fhIsSeekable =
-          exists( $_[0]->{seek} ) ? $_[0]->{seek} : _isSeekable($fh);
-    }
-    else {
-        $fh           = shift;
-        $fhIsSeekable = @_ ? shift : _isSeekable($fh);
-    }
-
+    my $fh   = shift;
     return _error('No filehandle given')   unless $fh;
     return _ioError('filehandle not open') unless $fh->opened();
+
+    my $fhIsSeekable = @_ ? shift: _isSeekable($fh);
     _binmode($fh);
 
     # Find out where the current position is.
     my $offset = $fhIsSeekable ? $fh->tell() : 0;
-    $offset    = 0 if $offset < 0;
+    $offset = 0 if $offset < 0;
 
     foreach my $member ( $self->members() ) {
         my $retval = $member->_writeToFileHandle( $fh, $fhIsSeekable, $offset );
@@ -453,7 +305,7 @@ sub overwrite {
 # Returns AZ_OK if successful.
 sub overwriteAs {
     my $self    = shift;
-    my $zipName = ( ref( $_[0] ) eq 'HASH' ) ? $_[0]->{filename} : shift;
+    my $zipName = shift;
     return _error("no filename in overwriteAs()") unless defined($zipName);
 
     my ( $fh, $tempName ) = Archive::Zip::tempFile();
@@ -510,7 +362,7 @@ sub _writeEOCDOffset {
 sub _writeEndOfCentralDirectory {
     my ( $self, $fh ) = @_;
 
-    $self->_print($fh, END_OF_CENTRAL_DIRECTORY_SIGNATURE_STRING)
+    $fh->print(END_OF_CENTRAL_DIRECTORY_SIGNATURE_STRING)
       or return _ioError('writing EOCD Signature');
     my $zipfileCommentLength = length( $self->zipfileComment() );
 
@@ -524,10 +376,10 @@ sub _writeEndOfCentralDirectory {
         $self->_writeCentralDirectoryOffset(),
         $zipfileCommentLength
     );
-    $self->_print($fh, $header)
+    $fh->print($header)
       or return _ioError('writing EOCD header');
     if ($zipfileCommentLength) {
-        $self->_print($fh,  $self->zipfileComment() )
+        $fh->print( $self->zipfileComment() )
           or return _ioError('writing zipfile comment');
     }
     return AZ_OK;
@@ -535,16 +387,7 @@ sub _writeEndOfCentralDirectory {
 
 # $offset can be specified to truncate a zip file.
 sub writeCentralDirectory {
-    my $self = shift;
-
-    my ( $fh, $offset );
-    if ( ref( $_[0] ) eq 'HASH' ) {
-        $fh     = $_[0]->{fileHandle};
-        $offset = $_[0]->{offset};
-    }
-    else {
-        ( $fh, $offset ) = @_;
-    }
+    my ( $self, $fh, $offset ) = @_;
 
     if ( defined($offset) ) {
         $self->{'writeCentralDirectoryOffset'} = $offset;
@@ -566,7 +409,7 @@ sub writeCentralDirectory {
 
 sub read {
     my $self     = shift;
-    my $fileName = ( ref( $_[0] ) eq 'HASH' ) ? shift->{filename} : shift;
+    my $fileName = shift;
     return _error('No filename given') unless $fileName;
     my ( $status, $fh ) = _newFileHandle( $fileName, 'r' );
     return _ioError("opening $fileName for read") unless $status;
@@ -580,17 +423,9 @@ sub read {
 }
 
 sub readFromFileHandle {
-    my $self = shift;
-
-    my ( $fh, $fileName );
-    if ( ref( $_[0] ) eq 'HASH' ) {
-        $fh       = $_[0]->{fileHandle};
-        $fileName = $_[0]->{filename};
-    }
-    else {
-        ( $fh, $fileName ) = @_;
-    }
-
+    my $self     = shift;
+    my $fh       = shift;
+    my $fileName = shift;
     $fileName = $fh unless defined($fileName);
     return _error('No filehandle given')   unless $fh;
     return _ioError('filehandle not open') unless $fh->opened();
@@ -734,23 +569,10 @@ sub _untaintDir {
 
 sub addTree {
     my $self = shift;
-
-    my ( $root, $dest, $pred, $compressionLevel );
-    if ( ref( $_[0] ) eq 'HASH' ) {
-        $root             = $_[0]->{root};
-        $dest             = $_[0]->{zipName};
-        $pred             = $_[0]->{select};
-        $compressionLevel = $_[0]->{compressionLevel};
-    }
-    else {
-        ( $root, $dest, $pred, $compressionLevel ) = @_;
-    }
-
-    return _error("root arg missing in call to addTree()")
-      unless defined($root);
+    my $root = shift or return _error("root arg missing in call to addTree()");
+    my $dest = shift;
     $dest = '' unless defined($dest);
-    $pred = sub { -r } unless defined($pred);
-
+    my $pred = shift || sub { -r };
     my @files;
     my $startDir = _untaintDir( cwd() );
 
@@ -763,19 +585,10 @@ sub addTree {
         local $main::_ = $File::Find::name;
         my $dir = _untaintDir($File::Find::dir);
         chdir($startDir);
-        if ( $^O eq 'MSWin32' && $Archive::Zip::UNICODE ) {
-            push( @files, Win32::GetANSIPathName($File::Find::name) ) if (&$pred);
-            $dir = Win32::GetANSIPathName($dir);
-        }
-        else {
-            push( @files, $File::Find::name ) if (&$pred);
-        }
+        push( @files, $File::Find::name ) if (&$pred);
         chdir($dir);
     };
 
-    if ( $^O eq 'MSWin32' && $Archive::Zip::UNICODE ) {
-        $root = Win32::GetANSIPathName($root);
-    }
     File::Find::find( $wanted, $root );
 
     my $rootZipName = _asZipDirName( $root, 1 );    # with trailing slash
@@ -784,13 +597,7 @@ sub addTree {
     $dest = _asZipDirName( $dest, 1 );              # with trailing slash
 
     foreach my $fileName (@files) {
-        my $isDir;
-        if ( $^O eq 'MSWin32' && $Archive::Zip::UNICODE ) {
-            $isDir = -d Win32::GetANSIPathName($fileName);
-        }
-        else {
-            $isDir = -d $fileName;
-        }
+        my $isDir = -d $fileName;
 
         # normalize, remove leading ./
         my $archiveName = _asZipDirName( $fileName, $isDir );
@@ -800,8 +607,6 @@ sub addTree {
         my $member = $isDir
           ? $self->addDirectory( $fileName, $archiveName )
           : $self->addFile( $fileName, $archiveName );
-        $member->desiredCompressionLevel($compressionLevel);
-
         return _error("add $fileName failed in addTree()") if !$member;
     }
     return AZ_OK;
@@ -809,27 +614,16 @@ sub addTree {
 
 sub addTreeMatching {
     my $self = shift;
-
-    my ( $root, $dest, $pattern, $pred, $compressionLevel );
-    if ( ref( $_[0] ) eq 'HASH' ) {
-        $root             = $_[0]->{root};
-        $dest             = $_[0]->{zipName};
-        $pattern          = $_[0]->{pattern};
-        $pred             = $_[0]->{select};
-        $compressionLevel = $_[0]->{compressionLevel};
-    }
-    else {
-        ( $root, $dest, $pattern, $pred, $compressionLevel ) = @_;
-    }
-
-    return _error("root arg missing in call to addTreeMatching()")
-      unless defined($root);
+    my $root = shift
+      or return _error("root arg missing in call to addTreeMatching()");
+    my $dest = shift;
     $dest = '' unless defined($dest);
-    return _error("pattern missing in call to addTreeMatching()")
-      unless defined($pattern);
+    my $pattern = shift
+      or return _error("pattern missing in call to addTreeMatching()");
+    my $pred = shift;
     my $matcher =
       $pred ? sub { m{$pattern} && &$pred } : sub { m{$pattern} && -r };
-    return $self->addTree( $root, $dest, $matcher, $compressionLevel );
+    return $self->addTree( $root, $dest, $matcher );
 }
 
 # $zip->extractTree( $root, $dest [, $volume] );
@@ -839,27 +633,11 @@ sub addTreeMatching {
 #
 sub extractTree {
     my $self = shift;
-
-    my ( $root, $dest, $volume );
-    if ( ref( $_[0] ) eq 'HASH' ) {
-        $root   = $_[0]->{root};
-        $dest   = $_[0]->{zipName};
-        $volume = $_[0]->{volume};
-    }
-    else {
-        ( $root, $dest, $volume ) = @_;
-    }
-
+    my $root = shift;    # Zip format
     $root = '' unless defined($root);
-    if ( defined $dest ) {
-        if ( $dest !~ m{/$} ) {
-            $dest .= '/';
-        }
-    }
-    else {
-        $dest = './';
-    }
-
+    my $dest = shift;    # Zip format
+    $dest = './' unless defined($dest);
+    my $volume  = shift;                              # optional
     my $pattern = "^\Q$root";
     my @members = $self->membersMatching($pattern);
 
@@ -878,16 +656,9 @@ sub extractTree {
 # Returns (possibly updated) member, if any; undef on errors.
 
 sub updateMember {
-    my $self = shift;
-
-    my ( $oldMember, $fileName );
-    if ( ref( $_[0] ) eq 'HASH' ) {
-        $oldMember = $_[0]->{memberOrZipName};
-        $fileName  = $_[0]->{name};
-    }
-    else {
-        ( $oldMember, $fileName ) = @_;
-    }
+    my $self      = shift;
+    my $oldMember = shift;
+    my $fileName  = shift;
 
     if ( !defined($fileName) ) {
         _error("updateMember(): missing fileName argument");
@@ -947,29 +718,18 @@ sub updateMember {
 # whether the file or directory already exists in the zip file.
 #
 # If the fourth argument $mirror is true, then delete all my members
-# if corresponding files were not found.
+# if corresponding files weren't found.
 
 sub updateTree {
     my $self = shift;
-
-    my ( $root, $dest, $pred, $mirror, $compressionLevel );
-    if ( ref( $_[0] ) eq 'HASH' ) {
-        $root             = $_[0]->{root};
-        $dest             = $_[0]->{zipName};
-        $pred             = $_[0]->{select};
-        $mirror           = $_[0]->{mirror};
-        $compressionLevel = $_[0]->{compressionLevel};
-    }
-    else {
-        ( $root, $dest, $pred, $mirror, $compressionLevel ) = @_;
-    }
-
-    return _error("root arg missing in call to updateTree()")
-      unless defined($root);
+    my $root = shift
+      or return _error("root arg missing in call to updateTree()");
+    my $dest = shift;
     $dest = '' unless defined($dest);
-    $pred = sub { -r } unless defined($pred);
-
     $dest = _asZipDirName( $dest, 1 );
+    my $pred = shift || sub { -r };
+    my $mirror = shift;
+
     my $rootZipName = _asZipDirName( $root, 1 );    # with trailing slash
     my $pattern = $rootZipName eq './' ? '^' : "^\Q$rootZipName\E";
 
@@ -1007,7 +767,6 @@ sub updateTree {
 
         $done{$memberName} = 1;
         my $changedMember = $self->updateMember( $memberName, $fileName );
-        $changedMember->desiredCompressionLevel($compressionLevel);
         return _error("updateTree failed to update $fileName")
           unless ref($changedMember);
     }
