@@ -6,14 +6,8 @@ use strict;
 use vars qw( $VERSION @ISA );
 
 BEGIN {
-    $VERSION = '1.31_04';
+    $VERSION = '1.21';
     @ISA     = qw( Archive::Zip );
-
-    if ( $^O eq 'MSWin32' ) {
-        require Win32;
-        require Encode;
-        Encode->import( qw{ decode_utf8 } );
-    }
 }
 
 use Archive::Zip qw(
@@ -25,7 +19,7 @@ use Archive::Zip qw(
 );
 
 use Time::Local ();
-use Compress::Raw::Zlib qw( Z_OK Z_STREAM_END MAX_WBITS );
+use Compress::Zlib qw( Z_OK Z_STREAM_END MAX_WBITS );
 use File::Path;
 use File::Basename;
 
@@ -51,52 +45,19 @@ sub _newFromZipFile {
 
 sub newFromString {
     my $class = shift;
-
-    my ( $stringOrStringRef, $fileName );
-    if ( ref( $_[0] ) eq 'HASH' ) {
-        $stringOrStringRef = $_[0]->{string};
-        $fileName          = $_[0]->{zipName};
-    }
-    else {
-        ( $stringOrStringRef, $fileName ) = @_;
-    }
-
-    my $self  = $class->STRINGMEMBERCLASS->_newFromString( $stringOrStringRef,
-        $fileName );
+    my $self  = $class->STRINGMEMBERCLASS->_newFromString(@_);
     return $self;
 }
 
 sub newFromFile {
     my $class = shift;
-
-    my ( $fileName, $zipName );
-    if ( ref( $_[0] ) eq 'HASH' ) {
-        $fileName = $_[0]->{fileName};
-        $zipName  = $_[0]->{zipName};
-    }
-    else {
-        ( $fileName, $zipName ) = @_;
-    }
-
-    my $self = $class->NEWFILEMEMBERCLASS->_newFromFileNamed( $fileName,
-      $zipName );
+    my $self  = $class->NEWFILEMEMBERCLASS->_newFromFileNamed(@_);
     return $self;
 }
 
 sub newDirectoryNamed {
     my $class = shift;
-
-    my ( $directoryName, $newName );
-    if ( ref( $_[0] ) eq 'HASH' ) {
-        $directoryName = $_[0]->{directoryName};
-        $newName       = $_[0]->{zipName};
-    }
-    else {
-        ( $directoryName, $newName ) = @_;
-    }
-
-    my $self  = $class->DIRECTORYMEMBERCLASS->_newNamed( $directoryName,
-        $newName );
+    my $self  = $class->DIRECTORYMEMBERCLASS->_newNamed(@_);
     return $self;
 }
 
@@ -120,9 +81,6 @@ sub new {
         'crc32'                    => 0,
         'compressedSize'           => 0,
         'uncompressedSize'         => 0,
-        'isSymbolicLink'           => 0,
-        'password'                 => undef,  # password for encrypted data
-        'crc32c'                   => -1,     # crc for decrypted data
         @_
     };
     bless( $self, $class );
@@ -147,15 +105,9 @@ sub versionMadeBy {
 }
 
 sub fileAttributeFormat {
-    my $self = shift;
-
-    if (@_) {
-        $self->{fileAttributeFormat} = ( ref( $_[0] ) eq 'HASH' )
-        ? $_[0]->{format} : $_[0];
-    }
-    else {
-        return $self->{fileAttributeFormat};
-    }
+    ( $#_ > 0 )
+      ? ( $_[0]->{'fileAttributeFormat'} = $_[1] )
+      : $_[0]->{'fileAttributeFormat'};
 }
 
 sub versionNeededToExtract {
@@ -163,29 +115,7 @@ sub versionNeededToExtract {
 }
 
 sub bitFlag {
-    my $self = shift;
-
-    # Set General Purpose Bit Flags according to the desiredCompressionLevel setting
-    if ( $self->desiredCompressionLevel == 1 || $self->desiredCompressionLevel == 2 ) {
-        $self->{'bitFlag'} = DEFLATING_COMPRESSION_FAST;
-    } elsif ( $self->desiredCompressionLevel == 3 || $self->desiredCompressionLevel == 4
-          || $self->desiredCompressionLevel == 5 || $self->desiredCompressionLevel == 6
-          || $self->desiredCompressionLevel == 7 ) {
-        $self->{'bitFlag'} = DEFLATING_COMPRESSION_NORMAL;
-    } elsif ( $self->desiredCompressionLevel == 8 || $self->desiredCompressionLevel == 9 ) {
-        $self->{'bitFlag'} = DEFLATING_COMPRESSION_MAXIMUM;
-    }
-
-    if ($Archive::Zip::UNICODE) {
-        $self->{'bitFlag'} |= 0x0800;
-    }
-    $self->{'bitFlag'};
-}
-
-sub password {
-    my $self = shift;
-    $self->{'password'} = shift if @_;
-    $self->{'password'};
+    shift->{'bitFlag'};
 }
 
 sub compressionMethod {
@@ -193,9 +123,8 @@ sub compressionMethod {
 }
 
 sub desiredCompressionMethod {
-    my $self = shift;
-    my $newDesiredCompressionMethod =
-      ( ref( $_[0] ) eq 'HASH' ) ? shift->{compressionMethod} : shift;
+    my $self                        = shift;
+    my $newDesiredCompressionMethod = shift;
     my $oldDesiredCompressionMethod = $self->{'desiredCompressionMethod'};
     if ( defined($newDesiredCompressionMethod) ) {
         $self->{'desiredCompressionMethod'} = $newDesiredCompressionMethod;
@@ -211,9 +140,8 @@ sub desiredCompressionMethod {
 }
 
 sub desiredCompressionLevel {
-    my $self = shift;
-    my $newDesiredCompressionLevel =
-      ( ref( $_[0] ) eq 'HASH' ) ? shift->{compressionLevel} : shift;
+    my $self                       = shift;
+    my $newDesiredCompressionLevel = shift;
     my $oldDesiredCompressionLevel = $self->{'desiredCompressionLevel'};
     if ( defined($newDesiredCompressionLevel) ) {
         $self->{'desiredCompressionLevel'}  = $newDesiredCompressionLevel;
@@ -262,20 +190,12 @@ sub externalFileAttributes {
 }
 
 # Convert UNIX permissions into proper value for zip file
-# Usable as a function or a method
+# NOT A METHOD!
 sub _mapPermissionsFromUnix {
-    my $self    = shift;
-    my $mode    = shift;
-    my $attribs = $mode << 16;
+    my $perms = shift;
+    return $perms << 16;
 
-    # Microsoft Windows Explorer needs this bit set for directories
-    if ( $mode & DIRECTORY_ATTRIB ) {
-        $attribs |= 16;
-    }
-
-    return $attribs;
-
-    # TODO: map more MS-DOS perms
+    # TODO: map MS-DOS perms too (RHSA?)
 }
 
 # Convert ZIP permissions into Unix ones
@@ -287,7 +207,7 @@ sub _mapPermissionsFromUnix {
 # See the mapattr() function in unix/unix.c
 # See the attribute format constants in unzpriv.h
 #
-# XXX Note that there's one situation that is not implemented
+# XXX Note that there's one situation that isn't implemented
 # yet that depends on the "extra field."
 sub _mapPermissionsToUnix {
     my $self = shift;
@@ -365,47 +285,30 @@ sub _mapPermissionsToUnix {
 
 sub unixFileAttributes {
     my $self     = shift;
-    my $oldPerms = $self->_mapPermissionsToUnix;
-
-    my $perms;
-    if ( @_ ) {
-        $perms = ( ref( $_[0] ) eq 'HASH' ) ? $_[0]->{attributes} : $_[0];
-
-        if ( $self->isDirectory ) {
+    my $oldPerms = $self->_mapPermissionsToUnix();
+    if (@_) {
+        my $perms = shift;
+        if ( $self->isDirectory() ) {
             $perms &= ~FILE_ATTRIB;
             $perms |= DIRECTORY_ATTRIB;
-        } else {
+        }
+        else {
             $perms &= ~DIRECTORY_ATTRIB;
             $perms |= FILE_ATTRIB;
         }
-        $self->{externalFileAttributes} = $self->_mapPermissionsFromUnix($perms);
+        $self->{'externalFileAttributes'} = _mapPermissionsFromUnix($perms);
     }
-
     return $oldPerms;
 }
 
 sub localExtraField {
-    my $self = shift;
-
-    if (@_) {
-        $self->{localExtraField} = ( ref( $_[0] ) eq 'HASH' )
-          ? $_[0]->{field} : $_[0];
-    }
-    else {
-        return $self->{localExtraField};
-    }
+    ( $#_ > 0 )
+      ? ( $_[0]->{'localExtraField'} = $_[1] )
+      : $_[0]->{'localExtraField'};
 }
 
 sub cdExtraField {
-    my $self = shift;
-
-    if (@_) {
-        $self->{cdExtraField} = ( ref( $_[0] ) eq 'HASH' )
-          ? $_[0]->{field} : $_[0];
-    }
-    else {
-        return $self->{cdExtraField};
-    }
+    ( $#_ > 0 ) ? ( $_[0]->{'cdExtraField'} = $_[1] ) : $_[0]->{'cdExtraField'};
 }
 
 sub extraFields {
@@ -414,15 +317,9 @@ sub extraFields {
 }
 
 sub fileComment {
-    my $self = shift;
-
-    if (@_) {
-        $self->{fileComment} = ( ref( $_[0] ) eq 'HASH' )
-          ? pack( 'C0a*', $_[0]->{comment} ) : pack( 'C0a*', $_[0] );
-    }
-    else {
-        return $self->{fileComment};
-    }
+    ( $#_ > 0 )
+      ? ( $_[0]->{'fileComment'} = pack( 'C0a*', $_[1] ) )
+      : $_[0]->{'fileComment'};
 }
 
 sub hasDataDescriptor {
@@ -456,14 +353,14 @@ sub uncompressedSize {
 }
 
 sub isEncrypted {
-    shift->{'bitFlag'} & GPBF_ENCRYPTED_MASK;
+    shift->bitFlag() & GPBF_ENCRYPTED_MASK;
 }
 
 sub isTextFile {
     my $self = shift;
     my $bit  = $self->internalFileAttributes() & IFA_TEXT_FILE_MASK;
     if (@_) {
-        my $flag = ( ref( $_[0] ) eq 'HASH' ) ? shift->{flag} : shift;
+        my $flag = shift;
         $self->{'internalFileAttributes'} &= ~IFA_TEXT_FILE_MASK;
         $self->{'internalFileAttributes'} |=
           ( $flag ? IFA_TEXT_FILE: IFA_BINARY_FILE );
@@ -485,78 +382,15 @@ sub isBinaryFile {
 
 sub extractToFileNamed {
     my $self = shift;
-
-    # local FS name
-    my $name = ( ref( $_[0] ) eq 'HASH' ) ? $_[0]->{name} : $_[0];
-    $self->{'isSymbolicLink'} = 0;
-
-    # Check if the file / directory is a symbolic link or not
-    if ( $self->{'externalFileAttributes'} == 0xA1FF0000 ) {
-        $self->{'isSymbolicLink'} = 1;
-        $self->{'newName'} = $name;
-        my ( $status, $fh ) = _newFileHandle( $name, 'r' );
-        my $retval = $self->extractToFileHandle($fh);
-        $fh->close();
-    } else {
-        #return _writeSymbolicLink($self, $name) if $self->isSymbolicLink();
-        
-        my ( $status, $fh );
-        if ( $^O eq 'MSWin32' && $Archive::Zip::UNICODE ) {
-            $name = decode_utf8( Win32::GetFullPathName($name) );
-            mkpath_win32($name);
-            if ( $self->{'compressedSize'} == 0 ) {
-                return;
-            }
-            else {
-                Win32::CreateFile($name);
-            }
-            ( $status, $fh ) = _newFileHandle( Win32::GetANSIPathName($name), 'w' );
-        }
-        else {
-            mkpath( dirname($name) );    # croaks on error
-            ( $status, $fh ) = _newFileHandle( $name, 'w' );
-        }
-        return _ioError("Can't open file $name for write") unless $status;
-        my $retval = $self->extractToFileHandle($fh);
-        $fh->close();
-        chmod ($self->unixFileAttributes(), $name)
-            or return _error("Can't chmod() ${name}: $!");
-        utime( $self->lastModTime(), $self->lastModTime(), $name );
-        return $retval;
-    }
-}
-
-sub mkpath_win32 {
-    my $path = shift;
-    use File::Spec;
-
-    my ( $volume, @path ) = File::Spec->splitdir($path);
-    $path = File::Spec->catfile( $volume, shift @path );
-    pop @path;
-    while (@path) {
-        $path = File::Spec->catfile( $path, shift @path );
-        Win32::CreateDirectory($path);
-    }
-}
-
-sub _writeSymbolicLink {
-    my $self = shift;
-    my $name = shift;
-    my $chunkSize = $Archive::Zip::ChunkSize;
-    #my ( $outRef, undef ) = $self->readChunk($chunkSize);
-    my $fh;
+    my $name = shift;    # local FS name
+    return _error("encryption unsupported") if $self->isEncrypted();
+    mkpath( dirname($name) );    # croaks on error
+    my ( $status, $fh ) = _newFileHandle( $name, 'w' );
+    return _ioError("Can't open file $name for write") unless $status;
     my $retval = $self->extractToFileHandle($fh);
-    my ( $outRef, undef ) = $self->readChunk(100);
-}
-
-sub isSymbolicLink {
-    my $self = shift;
-    if ( $self->{'externalFileAttributes'} == 0xA1FF0000 ) {
-        $self->{'isSymbolicLink'} = 1;
-    } else {
-        return 0;
-    }
-    1;
+    $fh->close();
+    utime( $self->lastModTime(), $self->lastModTime(), $name );
+    return $retval;
 }
 
 sub isDirectory {
@@ -601,22 +435,16 @@ sub _deflater {
 # Return the total size of my local header
 sub _localHeaderSize {
     my $self = shift;
-    {
-        use bytes;
-        return SIGNATURE_LENGTH + LOCAL_FILE_HEADER_LENGTH +
-          length( $self->fileName() ) + length( $self->localExtraField() );
-    }
+    return SIGNATURE_LENGTH + LOCAL_FILE_HEADER_LENGTH +
+      length( $self->fileName() ) + length( $self->localExtraField() );
 }
 
 # Return the total size of my CD header
 sub _centralDirectoryHeaderSize {
     my $self = shift;
-    {
-        use bytes;
-        return SIGNATURE_LENGTH + CENTRAL_DIRECTORY_FILE_HEADER_LENGTH +
-          length( $self->fileName() ) + length( $self->cdExtraField() ) +
-          length( $self->fileComment() );
-    }
+    return SIGNATURE_LENGTH + CENTRAL_DIRECTORY_FILE_HEADER_LENGTH +
+      length( $self->fileName() ) + length( $self->cdExtraField() ) +
+      length( $self->fileComment() );
 }
 
 # DOS date/time format
@@ -648,7 +476,7 @@ sub _dosToUnixTime {
     return $time_t;
 }
 
-# Note, this is not exactly UTC 1980, it's 1980 + 12 hours and 1
+# Note, this isn't exactly UTC 1980, it's 1980 + 12 hours and 1
 # minute so that nothing timezoney can muck us up.
 my $safe_epoch = 315576060;
 
@@ -675,24 +503,6 @@ sub _unixToDosTime {
     return $dt;
 }
 
-sub head {
-    my ($self, $mode) = (@_, 0);
-
-    use bytes;
-    return pack LOCAL_FILE_HEADER_FORMAT,
-        $self->versionNeededToExtract(),
-        $self->{'bitFlag'},
-        $self->desiredCompressionMethod(),
-        $self->lastModFileDateTime(),
-        $self->crc32(),
-        $mode
-            ? $self->_writeOffset()     # compressed size
-            : $self->compressedSize(),  # may need to be re-written later
-        $self->uncompressedSize(),
-        length( $self->fileName() ),
-        length( $self->localExtraField() );
-}
-
 # Write my local header to a file handle.
 # Stores the offset to the start of the header in my
 # writeLocalHeaderRelativeOffset member.
@@ -702,20 +512,29 @@ sub _writeLocalFileHeader {
     my $fh   = shift;
 
     my $signatureData = pack( SIGNATURE_FORMAT, LOCAL_FILE_HEADER_SIGNATURE );
-    $self->_print($fh, $signatureData)
+    $fh->print($signatureData)
       or return _ioError("writing local header signature");
 
-    my $header = $self->head(0);
+    my $header = pack(
+        LOCAL_FILE_HEADER_FORMAT,
+        $self->versionNeededToExtract(),
+        $self->bitFlag(),
+        $self->desiredCompressionMethod(),
+        $self->lastModFileDateTime(),
+        $self->crc32(),
+        $self->compressedSize(),    # may need to be re-written later
+        $self->uncompressedSize(),
+        length( $self->fileName() ),
+        length( $self->localExtraField() )
+    );
 
-    $self->_print($fh, $header) or return _ioError("writing local header");
-
-    # Check for a valid filename or a filename equal to a literal `0'
-    if ( $self->fileName() || $self->fileName eq '0' ) {
-        $self->_print($fh, $self->fileName() )
+    $fh->print($header) or return _ioError("writing local header");
+    if ( $self->fileName() ) {
+        $fh->print( $self->fileName() )
           or return _ioError("writing local header filename");
     }
     if ( $self->localExtraField() ) {
-        $self->_print($fh, $self->localExtraField() )
+        $fh->print( $self->localExtraField() )
           or return _ioError("writing local extra field");
     }
 
@@ -728,16 +547,12 @@ sub _writeCentralDirectoryFileHeader {
 
     my $sigData =
       pack( SIGNATURE_FORMAT, CENTRAL_DIRECTORY_FILE_HEADER_SIGNATURE );
-    $self->_print($fh, $sigData)
+    $fh->print($sigData)
       or return _ioError("writing central directory header signature");
 
-    my ( $fileNameLength, $extraFieldLength, $fileCommentLength );
-    {
-        use bytes;
-        $fileNameLength    = length( $self->fileName() );
-        $extraFieldLength  = length( $self->cdExtraField() );
-        $fileCommentLength = length( $self->fileComment() );
-    }
+    my $fileNameLength    = length( $self->fileName() );
+    my $extraFieldLength  = length( $self->cdExtraField() );
+    my $fileCommentLength = length( $self->fileComment() );
 
     my $header = pack(
         CENTRAL_DIRECTORY_FILE_HEADER_FORMAT,
@@ -759,18 +574,18 @@ sub _writeCentralDirectoryFileHeader {
         $self->writeLocalHeaderRelativeOffset()
     );
 
-    $self->_print($fh, $header)
+    $fh->print($header)
       or return _ioError("writing central directory header");
     if ($fileNameLength) {
-        $self->_print($fh,  $self->fileName() )
+        $fh->print( $self->fileName() )
           or return _ioError("writing central directory header signature");
     }
     if ($extraFieldLength) {
-        $self->_print($fh,  $self->cdExtraField() )
+        $fh->print( $self->cdExtraField() )
           or return _ioError("writing central directory extra field");
     }
     if ($fileCommentLength) {
-        $self->_print($fh,  $self->fileComment() )
+        $fh->print( $self->fileComment() )
           or return _ioError("writing central directory file comment");
     }
 
@@ -793,7 +608,7 @@ sub _writeDataDescriptor {
         $self->uncompressedSize()
     );
 
-    $self->_print($fh, $header)
+    $fh->print($header)
       or return _ioError("writing data descriptor");
     return AZ_OK;
 }
@@ -810,9 +625,20 @@ sub _refreshLocalFileHeader {
         IO::Seekable::SEEK_SET )
       or return _ioError("seeking to rewrite local header");
 
-    my $header = $self->head(1);
+    my $header = pack(
+        LOCAL_FILE_HEADER_FORMAT,
+        $self->versionNeededToExtract(),
+        $self->bitFlag(),
+        $self->desiredCompressionMethod(),
+        $self->lastModFileDateTime(),
+        $self->crc32(),
+        $self->_writeOffset(),    # compressed size
+        $self->uncompressedSize(),
+        length( $self->fileName() ),
+        length( $self->localExtraField() )
+    );
 
-    $self->_print($fh, $header)
+    $fh->print($header)
       or return _ioError("re-writing local header");
     $fh->seek( $here, IO::Seekable::SEEK_SET )
       or return _ioError("seeking after rewrite of local header");
@@ -821,8 +647,7 @@ sub _refreshLocalFileHeader {
 }
 
 sub readChunk {
-    my $self = shift;
-    my $chunkSize = ( ref( $_[0] ) eq 'HASH' ) ? $_[0]->{chunkSize} : $_[0];
+    my ( $self, $chunkSize ) = @_;
 
     if ( $self->readIsDone() ) {
         $self->endRead();
@@ -839,7 +664,6 @@ sub readChunk {
     my ( $bytesRead, $status ) = $self->_readRawChunk( \$buffer, $chunkSize );
     return ( \$buffer, $status ) unless $status == AZ_OK;
 
-    $buffer && $self->isEncrypted and $buffer = $self->_decode($buffer);
     $self->{'readDataRemaining'} -= $bytesRead;
     $self->{'readOffset'} += $bytesRead;
 
@@ -857,7 +681,7 @@ sub readChunk {
 }
 
 # Read the next raw chunk of my data. Subclasses MUST implement.
-#   my ( $bytesRead, $status) = $self->_readRawChunk( \$buffer, $chunkSize );
+#	my ( $bytesRead, $status) = $self->_readRawChunk( \$buffer, $chunkSize );
 sub _readRawChunk {
     my $self = shift;
     return $self->_subclassResponsibility();
@@ -880,11 +704,11 @@ sub _copyChunk {
 # ( $outputRef, $status) = $self->_deflateChunk( \$buffer );
 sub _deflateChunk {
     my ( $self, $buffer ) = @_;
-    my ( $status ) = $self->_deflater()->deflate( $buffer, my $out );
+    my ( $out,  $status ) = $self->_deflater()->deflate($buffer);
 
     if ( $self->_readDataRemaining() == 0 ) {
         my $extraOutput;
-        ( $status ) = $self->_deflater()->flush($extraOutput);
+        ( $extraOutput, $status ) = $self->_deflater()->flush();
         $out .= $extraOutput;
         $self->endRead();
         return ( \$out, AZ_STREAM_END );
@@ -903,7 +727,7 @@ sub _deflateChunk {
 # ( $outputRef, $status) = $self->_inflateChunk( \$buffer );
 sub _inflateChunk {
     my ( $self, $buffer ) = @_;
-    my ( $status ) = $self->_inflater()->inflate( $buffer, my $out );
+    my ( $out,  $status ) = $self->_inflater()->inflate($buffer);
     my $retval;
     $self->endRead() unless $status == Z_OK;
     if ( $status == Z_OK || $status == Z_STREAM_END ) {
@@ -936,7 +760,7 @@ sub rewindData {
     if (    $self->compressionMethod() == COMPRESSION_STORED
         and $self->desiredCompressionMethod() == COMPRESSION_DEFLATED )
     {
-        ( $self->{'deflater'}, $status ) = Compress::Raw::Zlib::Deflate->new(
+        ( $self->{'deflater'}, $status ) = Compress::Zlib::deflateInit(
             '-Level'      => $self->desiredCompressionLevel(),
             '-WindowBits' => -MAX_WBITS(),                     # necessary magic
             '-Bufsize'    => $Archive::Zip::ChunkSize,
@@ -949,7 +773,7 @@ sub rewindData {
     elsif ( $self->compressionMethod() == COMPRESSION_DEFLATED
         and $self->desiredCompressionMethod() == COMPRESSION_STORED )
     {
-        ( $self->{'inflater'}, $status ) = Compress::Raw::Zlib::Inflate->new(
+        ( $self->{'inflater'}, $status ) = Compress::Zlib::inflateInit(
             '-WindowBits' => -MAX_WBITS(),               # necessary magic
             '-Bufsize'    => $Archive::Zip::ChunkSize,
             @_
@@ -1035,7 +859,8 @@ sub contents {
 
 sub extractToFileHandle {
     my $self = shift;
-    my $fh = ( ref( $_[0] ) eq 'HASH' ) ? shift->{fileHandle} : shift;
+    return _error("encryption unsupported") if $self->isEncrypted();
+    my $fh = shift;
     _binmode($fh);
     my $oldCompression = $self->desiredCompressionMethod(COMPRESSION_STORED);
     my $status         = $self->rewindData(@_);
@@ -1053,7 +878,7 @@ sub _writeToFileHandle {
     my $offset       = shift;
 
     return _error("no member name given for $self")
-      if $self->fileName() eq '';
+      unless $self->fileName();
 
     $self->{'writeLocalHeaderRelativeOffset'} = $offset;
     $self->{'wasWritten'}                     = 0;
@@ -1067,12 +892,6 @@ sub _writeToFileHandle {
             or $self->desiredCompressionMethod() == COMPRESSION_DEFLATED )
     );
 
-    # Set both compressedSize and uncompressedSize to 0 if either of them is 0
-    if ( $self->uncompressedSize == 0 || $self->uncompressedSize == 0 ) {
-        $self->{'compressedSize'}   = 0;
-        $self->{'uncompressedSize'} = 0;
-    }
-
     my $shouldWriteDataDescriptor =
       ( $headerFieldsUnknown and not $fhIsSeekable );
 
@@ -1105,30 +924,22 @@ sub _writeData {
     my $self    = shift;
     my $writeFh = shift;
 
-    # If symbolic link, just create one if the operating system is Linux, Unix, BSD or VMS
-    # TODO: Add checks for other operating systems
-    if ( $self->{'isSymbolicLink'} == 1 && $^O eq 'linux' ) {
-        my $chunkSize = $Archive::Zip::ChunkSize;
-        my ( $outRef, $status ) = $self->readChunk($chunkSize);
-        symlink $$outRef, $self->{'newName'};
-    } else {
-        return AZ_OK if ( $self->uncompressedSize() == 0 );
-        my $status;
-        my $chunkSize = $Archive::Zip::ChunkSize;
-        while ( $self->_readDataRemaining() > 0 ) {
-            my $outRef;
-            ( $outRef, $status ) = $self->readChunk($chunkSize);
-            return $status if ( $status != AZ_OK and $status != AZ_STREAM_END );
+    return AZ_OK if ( $self->uncompressedSize() == 0 );
+    my $status;
+    my $chunkSize = $Archive::Zip::ChunkSize;
+    while ( $self->_readDataRemaining() > 0 ) {
+        my $outRef;
+        ( $outRef, $status ) = $self->readChunk($chunkSize);
+        return $status if ( $status != AZ_OK and $status != AZ_STREAM_END );
 
-            if ( length($$outRef) > 0 ) {
-                $self->_print($writeFh, $$outRef)
-                  or return _ioError("write error during copy");
-            }
-
-            last if $status == AZ_STREAM_END;
+        if ( length($$outRef) > 0 ) {
+            $writeFh->print($$outRef)
+              or return _ioError("write error during copy");
         }
-        $self->{'compressedSize'} = $self->_writeOffset();
+
+        last if $status == AZ_STREAM_END;
     }
+    $self->{'compressedSize'} = $self->_writeOffset();
     return AZ_OK;
 }
 
@@ -1137,138 +948,4 @@ sub _usesFileNamed {
     return 0;
 }
 
-# ##############################################################################
-#
-# Decrypt section
-#
-# H.Merijn Brand (Tux) 2011-06-28
-#
-# ##############################################################################
-
-# This code is derived from the crypt source of unzip-6.0 dated 05 Jan 2007
-# Its license states:
-#
-# --8<---
-# Copyright (c) 1990-2007 Info-ZIP.  All rights reserved.
-
-# See the accompanying file LICENSE, version 2005-Feb-10 or later
-# (the contents of which are also included in (un)zip.h) for terms of use.
-# If, for some reason, all these files are missing, the Info-ZIP license
-# also may be found at:  ftp://ftp.info-zip.org/pub/infozip/license.html
-#
-# crypt.c (full version) by Info-ZIP.      Last revised:  [see crypt.h]
-
-# The main encryption/decryption source code for Info-Zip software was
-# originally written in Europe.  To the best of our knowledge, it can
-# be freely distributed in both source and object forms from any country,
-# including the USA under License Exception TSU of the U.S. Export
-# Administration Regulations (section 740.13(e)) of 6 June 2002.
-
-# NOTE on copyright history:
-# Previous versions of this source package (up to version 2.8) were
-# not copyrighted and put in the public domain.  If you cannot comply
-# with the Info-Zip LICENSE, you may want to look for one of those
-# public domain versions.
-#
-# This encryption code is a direct transcription of the algorithm from
-# Roger Schlafly, described by Phil Katz in the file appnote.txt.  This
-# file (appnote.txt) is distributed with the PKZIP program (even in the
-# version without encryption capabilities).
-# -->8---
-
-# As of January 2000, US export regulations were amended to allow export
-# of free encryption source code from the US.  As of June 2002, these
-# regulations were further relaxed to allow export of encryption binaries
-# associated with free encryption source code.  The Zip 2.31, UnZip 5.52
-# and Wiz 5.02 archives now include full crypto source code.  As of the
-# Zip 2.31 release, all official binaries include encryption support; the
-# former "zcr" archives ceased to exist.
-# (Note that restrictions may still exist in other countries, of course.)
-
-# For now, we just support the decrypt stuff
-# All below methods are supposed to be private
-
-# use Data::Peek;
-
-my @keys;
-my @crct = do {
-    my $xor = 0xedb88320;
-    my @crc = (0) x 1024;
-
-    # generate a crc for every 8-bit value
-    foreach my $n (0 .. 255) {
-        my $c = $n;
-        $c = $c & 1 ? $xor ^ ($c >> 1) : $c >> 1 for 1 .. 8;
-        $crc[$n] = _revbe ($c);
-        }
-
-    # generate crc for each value followed by one, two, and three zeros */
-    foreach my $n (0 .. 255) {
-        my $c = ($crc[($crc[$n] >> 24) ^ 0] ^ ($crc[$n] << 8)) & 0xffffffff;
-        $crc[$_ * 256 + $n] = $c for 1 .. 3;
-        }
-    map { _revbe ($crc[$_]) } 0 .. 1023;
-    };
-
-sub _crc32
-{
-    my ($c, $b) = @_;
-    return ($crct[($c ^ $b) & 0xff] ^ ($c >> 8));
-    } # _crc32
-
-sub _revbe
-{
-    my $w = shift;
-    return (($w >> 24) + (($w >> 8) & 0xff00) +
-        (($w & 0xff00) << 8) + (($w & 0xff) << 24));
-    } # _revbe
-
-sub _update_keys
-{
-    use integer;
-    my $c = shift; # signed int
-    $keys[0] = _crc32 ($keys[0], $c);
-    $keys[1] = (($keys[1] + ($keys[0] & 0xff)) * 0x08088405 + 1) & 0xffffffff;
-    my $keyshift = $keys[1] >> 24;
-    $keys[2] = _crc32 ($keys[2], $keyshift);
-    } # _update_keys
-
-sub _zdecode ($)
-{
-    my $c = shift;
-    my $t = ($keys[2] & 0xffff) | 2;
-    _update_keys ($c ^= ((($t * ($t ^ 1)) >> 8) & 0xff));
-    return $c;
-    } # _zdecode
-
-sub _decode
-{
-    my $self = shift;
-    my $buff = shift;
-
-    $self->isEncrypted or return $buff;
-
-    my $pass = $self->password;
-    defined $pass or return "";
-
-    @keys = (0x12345678, 0x23456789, 0x34567890);
-    _update_keys ($_) for unpack "C*", $pass;
-    # DDumper { uk => [ @keys ] };
-
-    my $head = substr $buff, 0, 12, "";
-    my @head = map { _zdecode ($_) } unpack "C*", $head;
-    my $x = $self->{externalFileAttributes}
-        ?  ($self->{lastModFileDateTime} >> 8) & 0xff
-        :   $self->{crc32} >> 24;
-    $head[-1] == $x or return "";       # Password fail
-
-    # Worth checking ...
-    $self->{crc32c} = (unpack LOCAL_FILE_HEADER_FORMAT, pack "C*", @head)[3];
-
-    # DHexDump ($buff);
-    $buff = pack "C*" => map { _zdecode ($_) } unpack "C*" => $buff;
-    # DHexDump ($buff);
-    return $buff;
-    } # _decode
-
 1;
